﻿using HarmonyLib;
using System;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;

public class BatteryEqualizer : Mod
{
    static Harmony harmony = new Harmony("com.yasminendusa.batteryequalizer");
    public static List<Battery> batteries = new List<Battery>();
    static System.Timers.Timer tickTimer;
    static int powerBuffer;

    public void Start()
    {
        harmony.PatchAll(Assembly.GetExecutingAssembly());

        tickTimer = new System.Timers.Timer
        {
            Interval = 1000,
            AutoReset = true
        };

        tickTimer.Elapsed += TickTimer_Elapsed;
        tickTimer.Start();

        Debug.Log("Mod Battery Equalizer has been loaded!");
    }

    private void TickTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
    {
        try
        {
            Tick();
        }
        catch (System.Exception)
        {
        }
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll(harmony.Id);

        if (tickTimer != null)
        {
            if (tickTimer.Enabled)
            {
                tickTimer.Stop();
            }

            tickTimer.Dispose();
            tickTimer = null;
        }

        Debug.Log("Mod Battery Equalizer has been unloaded!");
    }

    public override void WorldEvent_WorldLoaded()
    {
        base.WorldEvent_WorldLoaded();
    }

    public override void WorldEvent_WorldSaved()
    {
        base.WorldEvent_WorldSaved();
    }

    private static void Tick()
    {
        if (Raft_Network.IsHost)
        {
            int allCharges = 0;
            int usableBatteryCount = 0;

            foreach (Battery battery in batteries)
            {
                if (battery != null && !battery.BatterySlotIsEmpty)
                {
                    allCharges += battery.BatteryUses;
                    usableBatteryCount++;
                }
            }

            if (usableBatteryCount > 0)
            {
                allCharges += powerBuffer;
                int levelValue = allCharges / usableBatteryCount;
                powerBuffer = allCharges % usableBatteryCount;

                Network_Player player = RAPI.GetLocalPlayer();
                if (player != null && batteries != null && batteries.Count > 0)
                {
                    //TODO show buffer somewhere on screen
                }

                foreach (Battery battery in batteries)
                {
                    if (battery != null && !battery.BatterySlotIsEmpty)
                    {
                        if (battery.BatteryUses > levelValue)
                        {
                            battery.RemoveBatteryUsesNetworked(battery.BatteryUses - levelValue);
                        }
                        else if (battery.BatteryUses < levelValue)
                        {
                            battery.AddBatteryUsesNetworked(levelValue - battery.BatteryUses);
                        }
                    }
                }
            }
        }
    }
}

[HarmonyPatch(typeof(Battery), "Start")]
public class HarmonyPatch_Battery_Start
{
    [HarmonyPostfix]
    static void Patch_Start(Battery __instance)
    {
        if (__instance == null)
            return;

        BatteryEqualizer.batteries.Add(__instance);
        Debug.Log("Battery added to the equalizer network. Index: " + __instance.BatteryIndex.ToString());
    }
}

[HarmonyPatch(typeof(UnityEngine.Object), "Destroy", new Type[] { typeof(UnityEngine.Object) })]
public class HarmonyPatch_Battery_Destroy
{
    [HarmonyPostfix]
    static void Patch_Destroy(UnityEngine.Object __instance)
    {
        if (__instance == null)
            return;

        if (__instance.GetType() == typeof(Battery))
        {
            if (BatteryEqualizer.batteries.Contains((Battery)__instance))
            {
                BatteryEqualizer.batteries.Remove((Battery)__instance);
                Debug.Log("Battery removed from the equalizer network. Index: " + ((Battery)__instance).BatteryIndex.ToString());
            }
        }
    }
}